using System;
using Syncfusion.Windows.Forms.Grid;
using System.Windows.Forms;
using System.Data;
using System.Diagnostics;

namespace SIS.Decide.DAWizard
{
	/// <summary>
	/// helps with paste operations for Syncfusion DataBound grid
	/// </summary>
	/// <remarks>Author Sahon</remarks>
	public class DataBoundGridPasteHelper
	{
		public static bool Paste(GridDataBoundGrid dtg, DataObject dataObject, out bool isError)
		{
			isError=false;
			if(!dataObject.GetDataPresent(DataFormats.Text)) return false;
			
			string s = (string)dataObject.GetData(DataFormats.Text);
			string[] rows = s.Split(new char[]{'\n'});
			int numRows = rows.GetLength(0);
			if(numRows > 0 && rows[numRows - 1].Length == 0)
				numRows--; //remove extra empty row if present	
			//add rows if needed...
			if(dtg.Binder.IsAddNew && numRows <= 1)
				return false;

			dtg.BeginUpdate();
			dtg.Binder.SuspendBinding();

			GridCurrentCell cc = dtg.CurrentCell;
			int rowIndex = cc.RowIndex;
			int oldRowIndex=rowIndex;
			int oldRowCount = dtg.Model.RowCount;
			int oldColIndex=cc.ColIndex;
			int extraRowIfStartAtAddNewRow = dtg.Binder.IsAddNew ? 1 : 0;
			int i=0;int j;
			string ss="";
			try
			{
				while (i<numRows)
				{
					if (i>oldRowCount-oldRowIndex-1-extraRowIfStartAtAddNewRow)
					{
						#region new
						j=rows[i].IndexOf("\t");
						if (j==-1) j=rows[i].IndexOf("\r");
						if (j!=-1) 
						{
							int columnIndex=oldColIndex, startPosition=0;
							bool isEnd=false;
							DataTable dt=dtg.DataSource as DataTable;
							if (dt!=null)
							{
								DataRow dr=dt.NewRow();
								while (!isEnd)
								{
									ss=rows[i].Substring(startPosition,j-startPosition);
									if (!SetValueDataRow(dr,dtg,ss,DBNull.Value,columnIndex)) isError=true;
									columnIndex++;
									startPosition=j+1;
									j=rows[i].IndexOf("\t",startPosition);
									if (j==-1) j=rows[i].IndexOf("\r",startPosition);
									isEnd=(j==-1 || columnIndex>dtg.Model.ColCount);
								}
								dt.Rows.Add(dr);
							}
						}
						#endregion new
					}
					else
					{
						#region old
						j=rows[i].IndexOf("\t");
						if (j==-1) j=rows[i].IndexOf("\r");
						if (j!=-1) 
						{
							int columnIndex=oldColIndex, startPosition=0;
							bool isEnd=false;
							while (!isEnd)
							{
								ss=rows[i].Substring(startPosition,j-startPosition);
								if (!SetValueGridStyleInfo(dtg[rowIndex,columnIndex],ss,System.DBNull.Value)) isError=true;;
								columnIndex++;
								startPosition=j+1;
								j=rows[i].IndexOf("\t",startPosition);
								if (j==-1) j=rows[i].IndexOf("\r",startPosition);
								isEnd=(j==-1 || columnIndex>dtg.Model.ColCount);
							}
						}
						#endregion old
					}
					i++;
					rowIndex++;
				}
				return true;
			}
			catch (Exception ex)
			{
				Trace.WriteLine(ex.ToString());
				throw new ApplicationException("There were errors during paste operations: " + ex.Message);
			}
			finally
			{
				dtg.Binder.ResumeBinding();
				dtg.EndUpdate();
				dtg.Refresh();
			}
		}
		private static bool SetValueGridStyleInfo(GridStyleInfo st, string val, object defaultValue)
		{
			bool isSet=false;
			if (st.CellType=="ComboBox")
			{
				DataTable dt=st.DataSource as DataTable;
				if (dt!=null)
				{
					foreach (DataRow dr in dt.Rows)
					{
						if ((string)dr[st.DisplayMember]==val)
						{
							st.Text=Convert.ToString(dr[st.ValueMember]);
							isSet=true;
							break;
						}
					}
					if (!isSet) st.CellValue=defaultValue;
				}
			}
			else if (st.CellType=="TextBox")
			{
				st.Text=val;
				isSet=true;
			}
			else if (st.CellType=="CheckBox")
			{
				st.CellValue=Convert.ToBoolean(val);
				isSet=true;
			}
			return isSet;
		}
		private static bool SetValueDataRow(DataRow row, GridDataBoundGrid dataGrid, string val, object defaultValue, int columnIndex)
		{
			bool isSet=false;
			string mappingName=dataGrid.Binder.InternalColumns[columnIndex-1].MappingName;
			if (mappingName=="") return true;
			DataTable dt=row.Table;
			if (dataGrid.Binder.InternalColumns[columnIndex-1].StyleInfo.CellType=="TextBox")
			{
				row[mappingName]=Convert.ChangeType(val, dt.Columns[mappingName].DataType);
				isSet=true;
			}
			if (dataGrid.Binder.InternalColumns[columnIndex-1].StyleInfo.CellType=="CheckBox")
			{
				row[mappingName]=Convert.ChangeType(val, dt.Columns[mappingName].DataType);
				isSet=true;
			}
			else if (dataGrid.Binder.InternalColumns[columnIndex-1].StyleInfo.CellType=="ComboBox")
			{
				string displayMember=dataGrid.Binder.InternalColumns[columnIndex-1].StyleInfo.DisplayMember;
				string valueMember=dataGrid.Binder.InternalColumns[columnIndex-1].StyleInfo.ValueMember;
				DataTable ds=dataGrid.Binder.InternalColumns[columnIndex-1].StyleInfo.DataSource as DataTable;
				if (ds!=null)
				{
					foreach (DataRow dr in ds.Rows)
					{
						if ((string)dr[displayMember]==val)
						{
							row[mappingName]=Convert.ChangeType(dr[valueMember], dt.Columns[mappingName].DataType);
							isSet=true;
							break;
						}
					}
					if (!isSet) row[mappingName]=defaultValue;
				}
			
			}
			return isSet;
		}
	}
}
